-- Smart Scoring
-- A module that allows custom scoring methods for timing modes.

-- This is separated into three categories, Score, Percent and Grade.
-- Each represents a method of scoring performed by the engine.
-- Score is the regular - (usually) 9 digit - score.
-- Percent is your percentage that you see on the score.
-- and Grade is what actually gets rewarded by the engine to the player.

local ScoringTable = {
    ["HoldNoteScore_Held"] = 0,
    ["HoldNoteScore_MissedHold"] = 0,
    ["HoldNoteScore_LetGo"] = 0,
    ["TapNoteScore_HitMine"] = 0,
	["TapNoteScore_AvoidMine"] = 0,
	["TapNoteScore_CheckpointMiss"] = 0,
	["TapNoteScore_Miss"] = 0,
	["TapNoteScore_ProW5"] = 0,
	["TapNoteScore_ProW4"] = 0,
	["TapNoteScore_ProW3"] = 0,
	["TapNoteScore_ProW2"] = 0,
	["TapNoteScore_ProW1"] = 0,
	["TapNoteScore_W5"] = 0,
	["TapNoteScore_W4"] = 0,
	["TapNoteScore_W3"] = 5,
	["TapNoteScore_W2"] = GAMESTATE:ShowW1() and 9 or 10,
	["TapNoteScore_W1"] = 10,
    ["TapNoteScore_MaxScore"] = 10,
	["TapNoteScore_CheckpointHit"] = 0
}

local PercentTable = {
    ["HoldNoteScore_Held"] = IsGame("pump") and 0 or 3,
    ["HoldNoteScore_MissedHold"] = 0,
    ["HoldNoteScore_LetGo"] = 0,
    ["TapNoteScore_HitMine"] = -2,
	["TapNoteScore_AvoidMine"] = 0,
	["TapNoteScore_CheckpointMiss"] = 0,
	["TapNoteScore_Miss"] = 0,
	["TapNoteScore_ProW5"] = 0,
	["TapNoteScore_ProW4"] = 0,
	["TapNoteScore_ProW3"] = 0,
	["TapNoteScore_ProW2"] = 0,
	["TapNoteScore_ProW1"] = 0,
	["TapNoteScore_W5"] = 0,
	["TapNoteScore_W4"] = 0,
	["TapNoteScore_W3"] = 1,
	["TapNoteScore_W2"] = GAMESTATE:ShowW1() and 2 or 3,
	["TapNoteScore_W1"] = 3,
    ["TapNoteScore_MaxScore"] = 3,
	["TapNoteScore_CheckpointHit"] = 3
}

local GradeTable = {
    ["HoldNoteScore_Held"] = IsGame("pump") and 0 or 3,
    ["HoldNoteScore_MissedHold"] = 0,
    ["HoldNoteScore_LetGo"] = 0,
    ["TapNoteScore_HitMine"] = -2,
	["TapNoteScore_AvoidMine"] = 0,
	["TapNoteScore_CheckpointMiss"] = 0,
	["TapNoteScore_Miss"] = 0,
	["TapNoteScore_ProW5"] = 0,
	["TapNoteScore_ProW4"] = 0,
	["TapNoteScore_ProW3"] = 0,
	["TapNoteScore_ProW2"] = 0,
	["TapNoteScore_ProW1"] = 0,
	["TapNoteScore_W5"] = 0,
	["TapNoteScore_W4"] = 0,
	["TapNoteScore_W3"] = 1,
	["TapNoteScore_W2"] = GAMESTATE:ShowW1() and 2 or 3,
	["TapNoteScore_W1"] = 3,
    ["TapNoteScore_MaxScore"] = 3,
	["TapNoteScore_CheckpointHit"] = 3
}

return Def.ActorFrame{
	InitCommand=function(self)
		-- Load any custom timing sets set by the current TimingWindow.
		-- This will be performed by overwriting existing values from the local tables above.
		self.mode = LoadModule("Config.Load.lua")("SmartTimings","Save/OutFoxPrefs.ini") or "Original"

		for k,v in pairs( TimingWindow ) do
			local TW = TimingWindow[k]()
            if self.mode == TW.Name then

				-- if there is a table that will share values with multiple sets, overwrite them.
				if( TW.Shared ) then
					for a,b in pairs( TW.Shared ) do
						ScoringTable[a] = b
						PercentTable[a] = b
						GradeTable[a] = b
					end
				end
                -- We found out table, time to replace settings.
				if( TW.Scoring ) then
					for a,b in pairs( TW.Scoring ) do ScoringTable[a] = b end
				end
				if( TW.Percent ) then
					for a,b in pairs( TW.Percent ) do PercentTable[a] = b end
				end
				if( TW.Grade ) then
					for a,b in pairs( TW.Grade ) do GradeTable[a] = b end
				end
                break
            end
        end
	end,
	ScoringWindowMessageCommand=function(self, params)
        params.score = ScoringTable[params.Type] or 0
    end,
    PercentWindowMessageCommand=function(self, params)
        params.Weight = PercentTable[params.Type] or 0
    end,
    GradeWindowMessageCommand=function(self, params)
        params.Weight = GradeTable[params.Type] or 0
    end
}